//
//  ByteSlice.h
//  HexFiend
//
//  Created by ridiculous_fish on 9/3/04.
//  Copyright 2004-2006 ridiculous_fish. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#include <sys/aio.h>


//a byte slice is an immutable array of bytes
@interface ByteSlice : NSObject <NSCopying> {
    unsigned long long myRetainCount;
}

- (unsigned long long)length;
- (void)copyBytes:(unsigned char*)dst range:(LongRange)range;
- (ByteSlice*)subsliceWithRange:(LongRange)range;

- (int)compareBytesAtOffset:(unsigned long long)thisOffset toBytesAtOffset:(unsigned long long)thatOffset ofSlice:(ByteSlice*)slice withLength:(unsigned long long)length;

//serialization estimation functions
- (unsigned long long)costToWrite;
- (unsigned long long)costToBreak;
- (unsigned long long)costToWriteOnceBroken;

@end

@interface ByteSlice (FastPath)

//attempts to do an optimized append, creating a new byte slice
//returns nil on failure
- (ByteSlice*)fastPathAppendByteSlice:(ByteSlice*)slice atLocation:(unsigned long long)location;

//returns a pointer to the internal memory of the ByteSlice, or NULL
- (const unsigned char*)bytesNoCopy;

@end

struct ByteSliceReadRequest {
    struct aiocb aiocb;
    ByteSlice *byteSlice;
    int error;
    unsigned length;
    BOOL completed;
};


/* Asynchronous IO routines - only implemented for file-type byte slices.  Look at bytesNoCopy, and if that returns nil then these should work. */
@interface ByteSlice (AIO)

- (BOOL)asyncCopyBytes:(unsigned char*)dst range:(LongRange)range request:(struct ByteSliceReadRequest *)request;
- (void)cancelRequest:(struct ByteSliceReadRequest*)request;
- (BOOL)joinRequest:(struct ByteSliceReadRequest*)request;

@end
