//#import "MyDocument.h"

#define NO_SELECTION ULLONG_MAX
#define FUZZY_SELECTION (ULLONG_MAX-1)
//fuzzy selection means that the anchor can be at either end of the selection,
//depending on which button on the keyboard you push next

#import "HexViewProtocol.h"

@class ByteArray, DataTextView, LineCountingTable;
@interface DataView : NSView {
	id <HexViewProtocol> myDocument; //not retained

	DataTextView* myTextView;
	
	BCFloat myAdditionalTextViewWidth;
	unsigned myIsCurrentlySettingSelection;
	unsigned mySuppressTextFrameChanges;
	unsigned mySuppressIBeamDrawing;
	unsigned mySuppressTextViewDisplaysForPerformance;
}

//returns the DataTextView subclass
+ textViewClass;

//returns the actual text view
- (DataTextView *)textView;

//returns the height of each row in the text view
- (double)rowHeight;

+ viewWithDocument:(id <HexViewProtocol>)document frame:(NSRect)frame;

- initWithDocument:(id <HexViewProtocol>)document frame:(NSRect)frame;

+ viewFromNibName:(NSString*)name document:(id <HexViewProtocol>)document frame:(NSRect)frame;

- (void)setDocument:(id <HexViewProtocol>)document;

- (id <HexViewProtocol>)document;
- (double)widthForFont:(NSFont *)font andBytesPerLine:(unsigned)bytesPerLine;

//returns the default size for the view
+ (NSSize)defaultSize;

//returns the default text font for the view
- (NSFont*)textFont;
//indicates that the font may have changed
- (void)updateFont;
//returns the height of each row in the text view
- (double)rowHeight;
//returns the number of visible lines for the view's current size
- (unsigned)visibleLines;
//converts the bytes in the given range to a string
- (NSString*)bytesToString:(LongRange)range;
//converts the string to bytes.  The more parameter is used to indicate
//whether there's incomplete data, e.g. a missing last nybble or part of a PowerPC instruction
- (NSData*)stringToBytes:(NSString*)string moreToCome:(BOOL*)more;

//converts the given data selection to a selected range in the text view
- (NSRange)dataSelectionToTextSelection:(LongRange)selection;

//converts the text selection to a data selection
- (LongRange)textSelectionToDataSelection:(NSRange)selection;

//updates the text selection in the text view
- (void)updateTextSelection;

//sets the attributed string to the selected bytes.  Must be implemented by subclasses.
- (void)setAttributedString:(NSMutableAttributedString*)string fromRange:(LongRange)range;

//updates the text view's text from the scroll value
- (void)refreshCurrentText;

//adds the text to the data at the given text range.  Must be implemented by subclasses.
//data is specific to the text view
- (void)insertText:(NSString*)text previousData:data;

//convenience method to return the document's selection
- (LongRange)selection;

//pasteboard writing
- (BOOL)writeSelectionToPasteboard:(NSPasteboard*)board type:(NSString *)type;
- (NSArray*)writablePasteboardTypes;

//pass NULL to use the general pboard
- (ByteArray*)bytesFromPasteboard:(NSPasteboard*)pboard;

- (void)paste:sender;

//called from the text view's equivalent method
- (BOOL)setTextSelection:(NSRange)range stillSelecting:(BOOL)flag;

//calls the text view's display method
- (void)forceTextViewDisplay;

//forgets any "pending" state (e.g. having typed the first nybble of a byte in hex mode)
- (void)forgetNextByteState;

- (void)selectAll:sender;

- (BOOL)suppressTextFrameChanges:view;
- (BOOL)suppressIBeamDrawing:view;
- (BOOL)isCurrentlySettingSelection;

- (void)setTextColor:(NSColor *)color;
@end

/*@interface NSObject (DataViewDelegate)
- (void)insertData:(NSData *)data backtrackingSelection:(int)backtracking;
- (NSFont *)hexFont;
- (unsigned long long)lineScroll;
- (unsigned)bytesPerLine;
- (unsigned long long)byteArrayLength;
- (ByteArray *)byteArray;
- (LongRange)selectionAnchorRange;
- (unsigned long long)selectionAnchorIndex;
- (LongRange)lselection;
- (BOOL)setDataSelection:(LongRange)dataSelection stillSelecting:(BOOL)flag fromDataView:(DataView *)dataView;
- (void)pasteBytes:(ByteArray *)bytes;
- (BOOL)dropBytes:(ByteArray *)bytes atIndex:(unsigned long long)index fromDataView:(DataView *)dataView removingOldSelection:(BOOL)deleteOld;
- (BOOL)setDataSelection:(LongRange)dataSelection stillSelecting:(BOOL)flag fromDataView:(DataView *)dataView;
- (void)moveDirection:(int)dir andModifySelection:(BOOL)modifySelection;
- (void)deleteDirection:(int)dir;
- (void)bulkMove:(int)move inDirection:(int)dir andModifySelection:(BOOL)flag;

@end*/
