#import <Cocoa/Cocoa.h>

//treats string as the hexadecimal representation of data
//converts 0-9, a-f, A-F to the corresponding nybble
//other characters are discarded
//if isMissingLastNybble is not NULL, sets *isMissingLastNybble to whether
//the last byte had only one nybble in it
NSData* hexStringToData(NSString* string, BOOL* isMissingLastNybble);


//returns c treated as a hex nybble, or -1 if it is not in [0-9, a-f, A-F]
int char2hex(unichar c);

//returns wall time as a number of seconds since the epoch (calls gettimeofday)
double fishTime(void);

static inline BOOL longRangesIntersect(LongRange a, LongRange b) {
	assert(a.location + a.length > a.location); //prevent overflows
	assert(b.location + b.length > b.location);
	
	return a.location < b.location + b.length && b.location < a.location + a.length;
}

static inline BOOL valueIsInLongRange(unsigned long long value, LongRange range) {
	return range.location <= value && value - range.location < range.length;
}

void setShouldCache(int fd, BOOL shouldCache);

//returns ULLONG_MAX if we can't figure it out
unsigned long long getAvailableSpace(int fd);
unsigned long long getCurrentFileSize(int fd);

/* returns an errno value on failure, 0 on success */
int truncateFileToSize(int fd, unsigned long long size);

NSString* describeByteCount(unsigned long long count);
NSString* describeByteCountWithPrefixAndSuffix(const char* prefix, unsigned long long count, const char* suffix);

/* returns the largest horizontal advancement for the glyphs represented by the characters in string in the given font */
double maximumAdvancementForGlyphs(NSFont *font, NSString *string);

/* gets the advancements for the glyphs represented by the characters in string.  Assumes that glyphCount = stringCount (turns off ligatures). */
void advancementsForGlyphs(NSFont *font, NSString *string, double *results);
